#ifndef HMATRIX_H
#define HMATRIX_H

#include <string>
#include <TROOT.h>
#include <TFile.h>
#include <TH1.h>

class HMatrixFile;

// ****************************************************************************
class HMatrix
{
public:

   HMatrix(Int_t id, const char* matrname, const HMatrixFile& file);
   ~HMatrix() { if (hist) delete hist; }

   void setXaxis(const char* name, const char* unit, Bool_t lower=kFALSE, Bool_t upper=kFALSE);
   void setYaxis(const char* name, const char* unit, Bool_t lower=kFALSE, Bool_t upper=kFALSE);
   void setZaxis(const char* name, const char* unit, Bool_t lower=kFALSE, Bool_t upper=kFALSE);

   Int_t getId() const { return id; }
   const char* getType() const { return type.c_str(); }
   void setType(const char* type) { type=type; }
   Bool_t getStatus() const { return status; }
   const char* getName() const { return name.c_str(); }
   Int_t getSize() const { return size; }

   Double_t operator()(Double_t x) const;
   Double_t operator()(Double_t x, Double_t y) const;
   Double_t operator()(Double_t x, Double_t y, Double_t z) const;

private:

   Int_t id;
   std::string type;

   std::string name;
   Bool_t status;
   TH1 *hist;
   Int_t size;

   std::string xname, yname, zname;
   std::string xunit, yunit, zunit;
   Bool_t extrLower[3], extrUpper[3];
   
   Int_t mapping[3];
   Double_t val[3];
   Double_t valscale[3];
   Double_t lower_limit[3];
   Double_t upper_limit[3];

   void setVal(Double_t x);
   void setVal(Double_t x, Double_t y);
   void setVal(Double_t x, Double_t y, Double_t z);

   static const Double_t kDeg2Rad;

ClassDef(HMatrix, 0)
};
// ****************************************************************************

#endif // HMATRIX_H
