// This macro filters data
// Date: 17.11.05 by WP

#include "hntuple.h"
#include "hmatrixparticle.h"
#include "hmatrixhistogram.h"
#include "hmatrixfilter.h"
#include <hpidgeanttrackset.h>
#include <algorithm>
#include <functional>
#include "hpidfl.h"


ClassImp(HMatrixFilter)




HGeantKine* HMatrixFilter::getGeantKine(Int_t iTrack, HCategory *pCat)//OK
{
// Return HGeantKine object corresponding to iTrack from pCat.
// If the pCat is not set (NULL) catGeantKine category from gHades is used.

    if(iTrack <= 0)
        return NULL;

    if((pCat == NULL) && ((pCat = HPidFL::getCategory(
                            catGeantKine, kFALSE)) == NULL))
    {
        ::Error("HPidGeantTrackSet::getGeantKine", "No catGeantKine category");

        return NULL;
    }

HGeantKine *pKine;
Int_t       iT, iMin, iMax, iC, iP;

    if((iMax = pCat->getEntries()) <= 0)
        return NULL;

    //check whether kine category has enough members at all to accomodate such a high track ID
    if((iMax > iTrack - 1)
            && (pKine = (HGeantKine *) pCat->getObject(iTrack - 1)) != NULL)
      {
        //iTrack is the geant track id! Those are sorted (well, they should be) and start with 1
        pKine->getParticle(iT, iP);

        //If the track number reported by the found kine object is the same that we ask for
        //this means that the ordering is done correctly in catGeantKine
        if(iT == iTrack)
          return pKine;
      }

    // fast searching for the proper track. The assumption: tracks are sorted!
    // implements a logarithmic search for the right object - very smart Mr. Jaskula :-)
    iMin = -1;

    do
    {
        iC = (iMax + iMin) / 2;
        if((pKine = (HGeantKine *) pCat->getObject(iC)) == NULL)
        {
            ::Error("HPidGeantTrackSet::getGeantKine",
                    "Hole in HGeantKine branch (%d)", iC);

            return NULL;
        }

        pKine->getParticle(iT, iP);
        if(iT == iTrack)
            return pKine;

        if(iTrack < iT)
            iMax = iC;
        else
            iMin = iC;
    } while(iMax - iMin > 1);

    return NULL;
}


	
//--------------------------------------------------------------
HMatrixFilter::HMatrixFilter(const char *output_file): HSUDummyRec(output_file) 
{
   ep_acc = new HMatrixHistogram();
   ep_eff = new HMatrixHistogram();
   em_acc = new HMatrixHistogram();
   em_eff = new HMatrixHistogram();
    p_acc = new HMatrixHistogram();
    p_eff = new HMatrixHistogram();
  pip_acc = new HMatrixHistogram();
  pip_eff = new HMatrixHistogram();
  pim_acc = new HMatrixHistogram();
  pim_eff = new HMatrixHistogram();

   m_weight = new HMatrixHistogram();
   pt_weight = new HMatrixHistogram();
   y_weight = new HMatrixHistogram();

   for (Int_t i=0; i<25; ++i)
   {
      epArr.push_back(new HMatrixParticle(2,"positron"));
      emArr.push_back(new HMatrixParticle(3,"electron"));
      pArr.push_back(new HMatrixParticle(14,"proton"));
      pipArr.push_back(new HMatrixParticle(8,"piplus"));
      pimArr.push_back(new HMatrixParticle(9,"piminus"));
   }
   epN = 0;
   emN = 0;
   pN = 0;
   pipN = 0;
   pimN = 0;

    // defining ntuple
    //Char_t *ntuple_vars = "id:mass:pt:y:oa:genInfo:genInfo1:genInfo2:ep_med:ep_mech:ep_par:em_med:em_mech:em_par:ep_acc:em_acc:ep_eff:em_eff:ep_mom:ep_momx:ep_momy:ep_momz:em_mom:em_momx:em_momy:em_momz:ep_theta:em_theta:ep_phi:em_phi:ep_parid:em_parid";
    //pNt = new HNtuple("dilep","dilep",ntuple_vars,320000);
    pNt = new HNtuple("dilep","dilep",320000);

      for (int i=0; i<15; ++i)
      {
	       fillSingle(epArr[i], i, 5);
      }
      for (int i=0; i<15; ++i)
      {
                fillSingle(emArr[i], i, 5);
      }
      for (int i=0; i<15; ++i)
      {
                fillSingle(pArr[i], i, 5);
      }

   //for (int i=0; i<5; ++i)
   //{
      //for (int j=0; j<5; ++j)
      //{
         //fillDouble(epArr[0], 55, emArr[0], 25);
      //}
   //}


   file_acc = 0;
   file_effem = 0;
   file_effep = 0;
   file_weight = 0;

   pGeantKineCat = 0;
   pitGeantKineCat1 = 0;
   pitGeantKineCat2 = 0;

   pKineHit1 = 0;
   pKineHit2 = 0;


   sim_execute = &HMatrixFilter::DEFAULT_execute;
}


//--------------------------------------------------------------
HMatrixFilter::~HMatrixFilter()
{
	//for_each(epArr.begin(), epArr.end(), del_fun<HMatrixParticle>());
	//for_each(emArr.begin(), epArr.end(), del_fun<HMatrixParticle>());
	//for_each(pArr.begin(), pArr.end(), del_fun<HMatrixParticle>());
	//for_each(pipArr.begin(), pipArr.end(), del_fun<HMatrixParticle>());
	//for_each(pimArr.begin(), pimArr.end(), del_fun<HMatrixParticle>());
}



//--------------------------------------------------------------
void HMatrixFilter::setSimInputType(const char* name)
{
   this->SetName(name);
   tempname = name;
   if (tempname.CompareTo("PLUTO")==0)
   {
      sim_execute = &HMatrixFilter::PLUTO_execute;
   }
   else if (tempname.CompareTo("UrQMD")==0)
   {
      sim_execute = &HMatrixFilter::UrQMD_execute;
   }
   else
   {
      sim_execute = &HMatrixFilter::DEFAULT_execute;
   }
}



//--------------------------------------------------------------
EXE_PTR HMatrixFilter::getSimInputType()
{
   return sim_execute;
}



//--------------------------------------------------------------
        void HMatrixFilter::setAccFile(const TString data)
        {
            acc_filename = data;
        }
        void HMatrixFilter::setEffFile(const TString data)
        {
            effep_filename = data;
            effem_filename = data;
        }
         void HMatrixFilter:: setEffFileEp(const TString data)
        {
            effep_filename = data;
        }
         void HMatrixFilter:: setEffFileEm(const TString data)
        {
            effem_filename = data;
        }
        void HMatrixFilter::setWeightFile(const TString data)
        {
            weight_filename = data;
        }


//--------------------------------------------------------------
        void HMatrixFilter::setEpAccName(const char* data)
        {
           ep_acc->SetName(data);
        }
        void HMatrixFilter::setEpAccXYZname(const char* x, const char* y, const char* z)
        {
           ep_acc->setXYZname(x, y, z);
        }
        void HMatrixFilter::setEpAccXYZunit(const char* x, const char* y, const char* z)
        {
           ep_acc->setXYZunit(x, y, z);
        }

        void HMatrixFilter::setEmAccName(const char* data)
        {
           em_acc->SetName(data);
        }
        void HMatrixFilter::setEmAccXYZname(const char* x, const char* y, const char* z)
        {
           em_acc->setXYZname(x, y, z);
        }
        void HMatrixFilter::setEmAccXYZunit(const char* x, const char* y, const char* z)
        {
           em_acc->setXYZunit(x, y, z);
        }

//--------------------------------------------------------------
        void HMatrixFilter::setEpEffName(const char* data)
        {
           ep_eff->SetName(data);
        }
        void HMatrixFilter::setEpEffXYZname(const char* x, const char* y, const char* z)
        {
           ep_eff->setXYZname(x, y, z);
        }
        void HMatrixFilter::setEpEffXYZunit(const char* x, const char* y, const char* z)
        {
           ep_eff->setXYZunit(x, y, z);
        }

        void HMatrixFilter::setEmEffName(const char* data)
        {
           em_eff->SetName(data);
        }
        void HMatrixFilter::setEmEffXYZname(const char* x, const char* y, const char* z)
        {
           em_eff->setXYZname(x, y, z);
        }
        void HMatrixFilter::setEmEffXYZunit(const char* x, const char* y, const char* z)
        {
           em_eff->setXYZunit(x, y, z);
        }

//--------------------------------------------------------------
        void HMatrixFilter::setMassName(const char* data)
        {
           m_weight->SetName(data);
        }
        void HMatrixFilter::setMassXname(const char* x)
        {
           m_weight->setXYZname(x);
        }
        void HMatrixFilter::setMassXunit(const char* x)
        {
           m_weight->setXYZunit(x);
        }

        void HMatrixFilter::setPtName(const char* data)
        {
           pt_weight->SetName(data);
        }
        void HMatrixFilter::setPtXname(const char* x)
        {
           pt_weight->setXYZname(x);
        }
        void HMatrixFilter::setPtXunit(const char* x)
        {
           pt_weight->setXYZunit(x);
        }

        void HMatrixFilter::setYName(const char* data)
        {
           y_weight->SetName(data);
        }
        void HMatrixFilter::setYXname(const char* x)
        {
           y_weight->setXYZname(x);
        }
        void HMatrixFilter::setYXunit(const char* x)
        {
           y_weight->setXYZunit(x);
        }




        
//--------------------------------------------------------------
        Bool_t HMatrixFilter::setAccHistos()
        {
           file_acc = NULL;
           if (acc_filename.Length()>0)
           {
              file_acc = new TFile(acc_filename);
              if (!file_acc->IsOpen())
              {
                 Error("setAccHistos",
                       "File '%s' not found.",acc_filename.Data());
                 return kFALSE;
              }
              ep_acc->setFile(file_acc);
              ep_acc->setHisto();

              em_acc->setFile(file_acc);
              em_acc->setHisto();
           } 
           else
           {
              Warning("setAccHistos","Filename of acceptance histograms not set.");
           }   
           if ((file_acc && !file_acc->IsOpen()) || 
               !ep_acc->getStatus() || !em_acc->getStatus()) return kFALSE;
           return kTRUE;
        }

//--------------------------------------------------------------
        Bool_t HMatrixFilter::setEffHistos()
        {
           file_effep = NULL;
           if (effep_filename.Length()>0)
           {
              file_effep = new TFile(effep_filename);
              if (!file_effep->IsOpen())
              {
                 Error("setEffHistos",
                       "File '%s' not found.",effep_filename.Data());
                 return kFALSE;
              }
              ep_eff->setFile(file_effep);
              ep_eff->setHisto();
           }
           else
           {
              Warning("setEffHistos","Filename of efficiency histograms not set.");
           }
            //eminus
            file_effem = NULL;
           if (effem_filename.Length()>0)
           {
              file_effem = new TFile(effem_filename);
              if (!file_effem->IsOpen())
              {
                 Error("setEffHistos",
                       "File '%s' not found.",effem_filename.Data());
                 return kFALSE;
              }

              em_eff->setFile(file_effem);
              em_eff->setHisto();
           }
           else
           {
              Warning("setEffHistos","Filename of efficiency histograms not set.");
           }

           if ((file_effep && !file_effep->IsOpen()) || 
               !ep_eff->getStatus() || !em_eff->getStatus()) return kFALSE;
           if ((file_effem && !file_effem->IsOpen()) || 
               !ep_eff->getStatus() || !em_eff->getStatus()) return kFALSE;

           return kTRUE;
        } 

//--------------------------------------------------------------
        Bool_t HMatrixFilter::setWeightHistos()
        {
           file_weight = NULL;
           if (weight_filename.Length()>0)
           {
              file_weight = new TFile(weight_filename);
              if (!file_weight->IsOpen())
              {
                 Error("setWeightHistos",
                       "File '%s' not found.",weight_filename.Data());
                 return kFALSE;
              }
              m_weight->setFile(file_weight);
              m_weight->setHisto();

              pt_weight->setFile(file_weight);
              pt_weight->setHisto();

              y_weight->setFile(file_weight);
              y_weight->setHisto();
           }
           else
           {
              Warning("setWeightHistos","Filename of weights histograms not set.");
           }

           if ((file_weight && !file_weight->IsOpen()) || 
               !m_weight->getStatus() || !pt_weight->getStatus() || !y_weight->getStatus()) return kFALSE;
           return kTRUE;
        }


//--------------------------------------------------------------
        Float_t HMatrixFilter::shiftPhi(const Float_t phi)
        {
        // shift of phi value from 0-360 deg range to one sector 0-60 range
        // necessary i.e. for acceptance matrices 

           Int_t sector = (Int_t)(phi/60.)-1;   // get sector from phi angle
           if (sector == -1) sector = 5;
           if (sector >= 0 && sector < 5)
           {
              return (phi - (sector+1)*60.); // we shift all to 0-60 deg. range
           }

           return phi;
        }


//---------------------------------------------------------------
Bool_t HMatrixFilter::isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2) 
{
  return (p1->getParent() == p2->getParent() && p1->getMechanism() == p2->getMechanism());
}

//---------------------------------------------------------------
Bool_t HMatrixFilter::isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3) 
{
  return (p1->getParent() == p2->getParent() == p3->getParent() && 
		  p1->getMechanism() == p2->getMechanism() == p3->getMechanism());
}

//---------------------------------------------------------------
Bool_t HMatrixFilter::isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3, const HMatrixParticle *p4) 
{
  return (p1->getParent() == p2->getParent() == p3->getParent() == p4->getParent() && 
		  p1->getMechanism() == p2->getMechanism() == p3->getMechanism() == p4->getMechanism());
}

//---------------------------------------------------------------
Bool_t HMatrixFilter::isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2)
{
  return (p1->getGenInfo1() == p2->getGenInfo1() && p1->getGenInfo2() == p2->getGenInfo2());
}

//---------------------------------------------------------------
Bool_t HMatrixFilter::isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3)
{
  return (p1->getGenInfo1() == p2->getGenInfo1() == p3->getGenInfo1() && 
          p1->getGenInfo2() == p2->getGenInfo2() == p3->getGenInfo2());
}

//---------------------------------------------------------------
Bool_t HMatrixFilter::isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3, const HMatrixParticle *p4)
{
  return (p1->getGenInfo1() == p2->getGenInfo1() == p3->getGenInfo1() == p4->getGenInfo1() && 
          p1->getGenInfo2() == p2->getGenInfo2() == p3->getGenInfo2() == p4->getGenInfo2());
}

//---------------------------------------------------------------
Float_t HMatrixFilter::getAcc(HMatrixParticle *p, HMatrixHistogram *h) 
{
   if (h->getStatus())
   {
      //Float_t a = h->getBinContent(p->getMom(), p->getTheta(), shiftPhi(p->getPhi()));
      //if (a>1. || a<0.)
	     //cout << "ACC ________________ " << a << " ||| " << p->getMom() <<", "<< p->getTheta() <<", "<<  shiftPhi(p->getPhi()) << endl;
      return h->getBinContent(p->getMom(), p->getTheta(), shiftPhi(p->getPhi()));
   }

return 0.;
}

//---------------------------------------------------------------
Float_t HMatrixFilter::getEff(HMatrixParticle *p, HMatrixHistogram *h)
{
   if (h->getStatus())
   {
      return h->getBinContent(p->getMom(), p->getTheta(), p->getPhi()-180.);
   }

return 0.;
}

//---------------------------------------------------------------
Float_t HMatrixFilter::getWeight(Float_t val, HMatrixHistogram *h)
{
   if (h->getStatus())
   {
      return h->getBinContent(val);
   }

return 0.;
}


HMatrixParticle* HMatrixFilter::getEp(Int_t i) { return epArr.at(i); } 
HMatrixParticle* HMatrixFilter::getEm(Int_t i) { return emArr.at(i); }
HMatrixParticle* HMatrixFilter::getP(Int_t i) { return pArr.at(i); }
HMatrixParticle* HMatrixFilter::getPip(Int_t i) { return pipArr.at(i); }
HMatrixParticle* HMatrixFilter::getPim(Int_t i) { return pimArr.at(i); }
Int_t HMatrixFilter::getEpN() { return epN; }
Int_t HMatrixFilter::getEmN() { return emN; }
Int_t HMatrixFilter::getPN() { return pN; }
Int_t HMatrixFilter::getPipN() { return pipN; }
Int_t HMatrixFilter::getPimN() { return pimN; }

Bool_t HMatrixFilter::isPLUTOprimary(HGeantKine *p)
{  
    Int_t par, med, mech;
    p->getCreator(par, med, mech);
    return (par == 0);
}  
   
Bool_t HMatrixFilter::isUrQMDprimary(HGeantKine *p)
{  
    Int_t par, med, mech;
    p->getCreator(par, med, mech);
    //return (med == 15 && (mech == 5 || mech == 6));
    
    if (par==0) return kTRUE;
       HGeantKine *pKineParent=getGeantKine(par);
    
    if (pKineParent)
       return pKineParent->isPrimary();
 return kFALSE;
}  


//---------------------------------------------------------------
Bool_t HMatrixFilter::init(void)
{
    setInitOk(kFALSE);
    setAccHistos();
    setEffHistos();
    setWeightHistos();

    
    TFile *ftemp = openOutFile();

    if(ftemp == NULL)
          return kFALSE;

    ftemp->cd();
    addObjectToWrite(pNt,kFALSE);
    //addObjectToWrite(pNt,kFALSE);

    // getting categories

    if((pGeantKineCat = static_cast<HLinearCategory*>(getCategory(catGeantKine))) == NULL)
    {
        Error("init", "Cannot get catGeantKine category.");
        goto lab_End;
    }

    if((pitGeantKineCat1 = getIterator(catGeantKine)) == NULL)
    {
        Error("init", "Cannot get catGeantKine iterator.");
        goto lab_End;
    }
    if((pitGeantKineCat2 = getIterator(catGeantKine)) == NULL)
    {
        Error("init", "Cannot get catGeantKine iterator.");
        goto lab_End;
    }

    setInitOk();

lab_End:

    cout << "INITIALISATION STATUS : "<<getInitOk()<<endl;
    ftemp->cd();
    return getInitOk();
}


//---------------------------------------------------------------
Bool_t HMatrixFilter::reinit(void)
{
   return kTRUE;
}

//---------------------------------------------------------------

Bool_t HMatrixFilter::finalize(void)
{
	// output file 
        writeObjects(getOutFile());
	return writeAndCloseOutFile();
}



//---------------------------------------------------------------
Int_t HMatrixFilter::execute()
{
   epN = 0;
   emN = 0;
   pN = 0;
   pipN = 0;
   pimN = 0; 
   return (this->*sim_execute)();
//return 0;
}




//---------------------------------------------------------------
Int_t HMatrixFilter::DEFAULT_execute()
{
   Warning("DEFAULT_execute", "No SIM input type defined. Dummy execute function called.");
return 0;
}





//---------------------------------------------------------------
Int_t HMatrixFilter::UrQMD_execute()
{


   epN = emN = pN = pipN = pimN = 0;

   pitGeantKineCat1->Reset();
   while((pKineHit1 = (HGeantKine*)pitGeantKineCat1->Next()) != 0)
   {
      if (isUrQMDprimary(pKineHit1) == kTRUE)
      {
         pKineHit2 = pKineHit1->getParent(pKineHit1->getTrack(), pGeantKineCat);
         parentid = 0;
         grandparentid = 0;
         grandgrandparentid = 0;
         if (pKineHit2 != 0)
         {
            parentid = pKineHit2->getID();
            HGeantKine *temp = pKineHit1->getGrandParent(pKineHit1->getTrack(), pGeantKineCat);
            if (temp != 0)
                 grandparentid = (pKineHit1->getGrandParent(pKineHit1->getTrack(), pGeantKineCat))->getID();
            temp = pKineHit1->getGrandParent(pKineHit2->getTrack(), pGeantKineCat);
            if (temp != 0)
                 grandgrandparentid = (pKineHit1->getGrandParent(pKineHit2->getTrack(), pGeantKineCat))->getID();
         }

         switch(pKineHit1->getID())
         {
             case 2:  *epArr[epN] = *pKineHit1;
                      epArr[epN]->setAcc( getAcc(epArr[epN], ep_acc) );
                      epArr[epN]->setEff( getEff(epArr[epN], ep_eff) );
                      epArr[epN]->Smear(4);
                      epArr[epN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
                      ++epN;
                      break;
             case 3:  *emArr[emN] = *pKineHit1;
                      emArr[emN]->setAcc( getAcc(emArr[emN], em_acc) );
		      //if (emArr[emN]->getAcc()>1. || emArr[emN]->getAcc()<0.) cout << "____________ poczas wypleniania _________- " << emArr[emN]->getAcc() << endl;
                      emArr[emN]->setEff( getEff(emArr[emN], em_eff) );
                      emArr[emN]->Smear(4);
                      emArr[emN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
                      ++emN;
                      break;
             case 8:  *pipArr[pipN] = *pKineHit1;
                      pipArr[pipN]->setAcc( getAcc(pipArr[pipN], pip_acc) );
                      pipArr[pipN]->setEff( getEff(pipArr[pipN], pip_eff) );
                      pipArr[pipN]->Smear(4);
                      pipArr[pipN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
                      ++pipN;
                      break;
             case 9:  *pimArr[pimN] = *pKineHit1;
                      pimArr[pimN]->setAcc( getAcc(pimArr[pimN], pim_acc) );
                      pimArr[pimN]->setEff( getEff(pimArr[pimN], pim_eff) );
                      pimArr[pimN]->Smear(4);
                      pimArr[pimN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
                      ++pimN;
                      break;
             case 14:  *pArr[pN] = *pKineHit1;
                      pArr[pN]->setAcc( getAcc(pArr[pN], p_acc) );
                      pArr[pN]->setEff( getEff(pArr[pN], p_eff) );
                      pArr[pN]->Smear(4);
                      pArr[pN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
                      ++pN;
                       break;
         }
      }
   }

   // ---------------------------------------- single ---
   for (int i=0; i<epN; ++i)
   {
      fillSingle(epArr[i], i, epN);
   }
   for (int i=0; i<emN; ++i)
   {
      fillSingle(emArr[i], i, emN);
   }
   for (int i=0; i<pN; ++i)
   {
      fillSingle(pArr[i], i, pN);
   }

   // ---------------------------------------- double ---
   for (int i=0; i<epN; ++i)
   {
      for (int j=0; j<emN; ++j)
      {
    //     fillDouble(epArr[i], i, emArr[j], j);

            for (int k=0; k<pN; ++k)
            {
    //           fillTriple(epArr[i], i, emArr[j], j, pArr[k], k);

               for (int m=0; m<pN; ++m)
               {
                  if (k<m)
                  {
     //                fillQuadruple(epArr[i], i, emArr[j], j, pArr[k], k, pArr[m], m);
                  }
               }
            }
      }
   }

   if ((epN+emN) > 0) pNt->fill();


return 0;
}








//---------------------------------------------------------------
Int_t HMatrixFilter::PLUTO_execute(void)
{


   epN = emN = pN = pipN = pimN = 0;

   pitGeantKineCat1->Reset();
   while((pKineHit1 = (HGeantKine*)pitGeantKineCat1->Next()) != 0)
   {
	  if (isPLUTOprimary(pKineHit1) == kTRUE)
	  {
         pKineHit2 = pKineHit1->getParent(pKineHit1->getTrack(), pGeantKineCat);
		 parentid = 0;
		 grandparentid = 0;
		 grandgrandparentid = 0;
		 if (pKineHit2 != 0)
		 {
            parentid = pKineHit2->getID();
            grandparentid = (pKineHit1->getGrandParent(pKineHit1->getTrack(), pGeantKineCat))->getID();
            grandgrandparentid = (pKineHit1->getGrandParent(pKineHit2->getTrack(), pGeantKineCat))->getID();
		 }

         switch(pKineHit1->getID())
	     {
			 case 2:  *epArr[epN] = *pKineHit1;
					  epArr[epN]->setAcc( getAcc(epArr[epN], ep_acc) );
					  epArr[epN]->setEff( getEff(epArr[epN], ep_eff) );
					  epArr[epN]->Smear(4);
                      epArr[epN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
					  ++epN;
					  break;
			 case 3:  *emArr[emN] = *pKineHit1;
					  emArr[emN]->setAcc( getAcc(emArr[emN], em_acc) );
					  emArr[emN]->setEff( getEff(emArr[emN], em_eff) );
					  emArr[emN]->Smear(4);
					  emArr[emN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
					  ++emN;
					  break;
			 case 8:  *pipArr[pipN] = *pKineHit1;
					  pipArr[pipN]->setAcc( getAcc(pipArr[pipN], pip_acc) );
					  pipArr[pipN]->setEff( getEff(pipArr[pipN], pip_eff) );
					  pipArr[pipN]->Smear(4);
					  pipArr[pipN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
					  ++pipN;
					  break;
			 case 9:  *pimArr[pimN] = *pKineHit1;
					  pimArr[pimN]->setAcc( getAcc(pimArr[pimN], pim_acc) );
					  pimArr[pimN]->setEff( getEff(pimArr[pimN], pim_eff) );
					  pimArr[pimN]->Smear(4);
					  pimArr[pimN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
					  ++pimN;
					  break;
			 case 14:  *pArr[pN] = *pKineHit1;
					  pArr[pN]->setAcc( getAcc(pArr[pN], p_acc) );
					  pArr[pN]->setEff( getEff(pArr[pN], p_eff) );
					  pArr[pN]->Smear(4);
					  pArr[pN]->setParGparGGparId(parentid,grandparentid,grandgrandparentid);
					  ++pN;
					   break;
	     }
	  }
   }

   // ---------------------------------------- single ---
   for (int i=0; i<epN; ++i)
   {
      fillSingle(epArr[i], i, epN);
   }
   for (int i=0; i<emN; ++i)
   {
      fillSingle(emArr[i], i, emN);
   }
   for (int i=0; i<pN; ++i)
   {
      fillSingle(pArr[i], i, pN);
   }
   

   // ---------------------------------------- double ---
   for (int i=0; i<epN; ++i)
   {
      for (int j=0; j<emN; ++j)
      {
         fillDouble(epArr[i], i, emArr[j], j);

            for (int k=0; k<pN; ++k)
            {
               fillTriple(epArr[i], i, emArr[j], j, pArr[k], k);

			   for (int m=0; m<pN; ++m)
			   {
			      if (k<m)
				  {
				     fillQuadruple(epArr[i], i, emArr[j], j, pArr[k], k, pArr[m], m);
				  }
			   }
            }
      }
   }

   pNt->fill();


return 0;
}


//---------------------------------------------------------------
void HMatrixFilter::fillSingle(HMatrixParticle *p, Int_t n, Int_t mult)
{
   string pName = p->getName() + to_string(n);
   HNtuple& href = *pNt;

   //pNt->setVal(pName + "_mult", mult);
   href[pName + "_mult"] = mult;
   pNt->setVal(pName + "_mass", p->getMass());
   pNt->setVal(pName + "_pt", p->getPt());
   pNt->setVal(pName + "_y", p->getY());
   href[pName + "_p"] = p->getMom();
   //pNt->setVal(pName + "_p", p->getMom());
   pNt->setVal(pName + "_px", p->getMomx());
   pNt->setVal(pName + "_py", p->getMomy());
   pNt->setVal(pName + "_pz", p->getMomz());
   pNt->setVal(pName + "_med", p->getMedium());
   pNt->setVal(pName + "_mech", p->getMedium());
   pNt->setVal(pName + "_par", p->getParent());
   pNt->setVal(pName + "_parid", p->getParentId());
   pNt->setVal(pName + "_gparid", p->getGrandparentId());
   pNt->setVal(pName + "_ggparid", p->getGrandgrandparentId());
   pNt->setVal(pName + "_theta", p->getTheta());
   pNt->setVal(pName + "_phi", p->getPhi());
   pNt->setVal(pName + "_geninfo", p->getGenInfo());
   pNt->setVal(pName + "_geninfo1", p->getGenInfo1());
   pNt->setVal(pName + "_geninfo2", p->getGenInfo2());
   pNt->setVal(pName + "_acc", p->getAcc());
   pNt->setVal(pName + "_eff", p->getEff());

   //cout << "SINGLE: " << pName <<endl;
   //cout << "SINGLE: " << pName << "("<<p->getMom()<<","<<p->getAcc()<<")"<<endl;
}

//---------------------------------------------------------------
void HMatrixFilter::fillDouble(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2)
{
   //string pName = p1->getName() + to_string(n1) + p2->getName() + to_string(n2);
   string pName = string(p1->getName()) + p2->getName();
   lv_a = *p1->getLVector();
   lv_b = *p2->getLVector();
   lv2 = lv_a + lv_b;
   opening_angle = ((180.0/TMath::Pi()) * lv_a.Angle(lv_b.Vect()));

   pNt->setVal(pName + "_vertex", isPLUTOvertex(p1, p2));
   pNt->setVal(pName + "_Uvertex", isUrQMDvertex(p1, p2));
   pNt->setVal(pName + "_mass", lv2.M());
   pNt->setVal(pName + "_pt", lv2.Pt());
   pNt->setVal(pName + "_y", lv2.Rapidity());
   pNt->setVal(pName + "_oa", opening_angle);
   pNt->setVal(pName + "_acc", p1->getAcc() * p2->getAcc());
   pNt->setVal(pName + "_eff", p1->getEff() * p2->getEff());
}

//---------------------------------------------------------------
void HMatrixFilter::fillTriple(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2, HMatrixParticle *p3, Int_t n3)
{
   string pName = p1->getName() + to_string(n1) + p2->getName() + to_string(n2) + p3->getName() + to_string(n3);
   lv_a = *p1->getLVector();
   lv_b = *p2->getLVector();
   lv_c = *p3->getLVector();
   lv3 = lv_a + lv_b + lv_c;

   Float_t oa9 = 0;
   if ( ((180.0/TMath::Pi()) * lv_a.Angle(lv_b.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_a.Angle(lv_c.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_b.Angle(lv_c.Vect())) > 9 )
     oa9 = 1;

   pNt->setVal(pName + "_vertex", isPLUTOvertex(p1, p2, p3));
   pNt->setVal(pName + "_Uvertex", isUrQMDvertex(p1, p2, p3));
   pNt->setVal(pName + "_mass", lv3.M());
   pNt->setVal(pName + "_pt", lv3.Pt());
   pNt->setVal(pName + "_y", lv3.Rapidity());
   pNt->setVal(pName + "_oa", oa9);
   pNt->setVal(pName + "_acc", p1->getAcc() * p2->getAcc() * p3->getAcc());
   pNt->setVal(pName + "_eff", p1->getEff() * p2->getEff() * p3->getEff());
}


//---------------------------------------------------------------
void HMatrixFilter::fillQuadruple(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2, HMatrixParticle *p3, Int_t n3, HMatrixParticle *p4, Int_t n4)
{
   string pName = p1->getName() + to_string(n1) + p2->getName() + to_string(n2) + p3->getName() + to_string(n3) + p4->getName() + to_string(n4);
   lv_a = *p1->getLVector();
   lv_b = *p2->getLVector();
   lv_c = *p3->getLVector();
   lv_d = *p4->getLVector();
   lv4 = lv_a + lv_b + lv_c + lv_d;

   Float_t oa9 = 0;
   if ( ((180.0/TMath::Pi()) * lv_a.Angle(lv_b.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_a.Angle(lv_c.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_a.Angle(lv_d.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_b.Angle(lv_c.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_b.Angle(lv_d.Vect())) > 9 &&
        ((180.0/TMath::Pi()) * lv_c.Angle(lv_d.Vect())) > 9 
      )
     oa9 = 1;

   pNt->setVal(pName + "_vertex", isPLUTOvertex(p1, p2, p3, p4));
   pNt->setVal(pName + "_Uvertex", isUrQMDvertex(p1, p2, p3, p4));
   pNt->setVal(pName + "_mass", lv4.M());
   pNt->setVal(pName + "_pt", lv4.Pt());
   pNt->setVal(pName + "_y", lv4.Rapidity());
   pNt->setVal(pName + "_oa", oa9);
   pNt->setVal(pName + "_acc", p1->getAcc() * p2->getAcc() * p3->getAcc() * p4->getAcc());
   pNt->setVal(pName + "_eff", p1->getEff() * p2->getEff() * p3->getEff() * p4->getEff());
}

