// This macro filters data
// Date: 17.11.05 by WP


#include <vector>
#include <sstream>
#include <TError.h>
#include <TVector.h>
#include <TLorentzVector.h>
#include <TString.h>
//#include <TNtuple.h>
#include "hsudummyrec.h"
#include "hiterator.h"
#include "hlinearcategory.h"
#include "TMath.h"
#include "hgeantkine.h"

template<class T>
struct del_fun_t
{
   del_fun_t& operator()(T* p) {
    delete p;
    return *this;
   }
};

template<class T>
del_fun_t<T> del_fun()
{
   return del_fun_t<T>();
}

template <class T>
inline std::string to_string (const T& t)
{
    std::stringstream ss;
    ss << t;
    return ss.str();
}


class HNtuple;
class HMatrixParticle;
class HMatrixHistogram;
class HMatrixFilter;


typedef Int_t (HMatrixFilter::*EXE_PTR)();

// ****************************************************************************
class HMatrixFilter: public HSUDummyRec 
{
public: 
	HMatrixFilter(const char *output_file);
    	~HMatrixFilter(void);

    virtual Bool_t init(void);
    virtual Bool_t reinit(void);
    virtual Int_t  execute(void);
    virtual Bool_t finalize(void);

    virtual Int_t DEFAULT_execute(void);
    virtual Int_t PLUTO_execute(void);
    virtual Int_t UrQMD_execute(void);


    virtual void setSimInputType(const char* name);
    EXE_PTR getSimInputType();

    void setAccFile(const TString data=NULL);
    void setEffFileEp(const TString data=NULL);
    void setEffFile(const TString data=NULL);
    void setEffFileEm(const TString data=NULL);
    void setWeightFile(const TString data=NULL);

    void setEpAccName(const char* data);
    void setEpAccXYZname(const char* x, const char* y, const char* y);
    void setEpAccXYZunit(const char* x, const char* y, const char* y);
    void setEmAccName(const char* data);
    void setEmAccXYZname(const char* x, const char* y, const char* y);
    void setEmAccXYZunit(const char* x, const char* y, const char* y);

    void setEpEffName(const char* data);
    void setEpEffXYZname(const char* x, const char* y, const char* y);
    void setEpEffXYZunit(const char* x, const char* y, const char* y);
    void setEmEffName(const char* data);
    void setEmEffXYZname(const char* x, const char* y, const char* y);
    void setEmEffXYZunit(const char* x, const char* y, const char* y);

    void setMassName(const char* data);
    void setMassXname(const char* x);
    void setMassXunit(const char* x);

    void setPtName(const char* data);
    void setPtXname(const char* x);
    void setPtXunit(const char* x);

    void setYName(const char* data);
    void setYXname(const char* x);
    void setYXunit(const char* x);

    Float_t getAcc(HMatrixParticle *p, HMatrixHistogram *h);
    Float_t getEff(HMatrixParticle *p, HMatrixHistogram *h);
    Float_t getWeight(Float_t val, HMatrixHistogram *h);

	HMatrixParticle* getEp(Int_t i);
	HMatrixParticle* getEm(Int_t i);
	HMatrixParticle* getP(Int_t i);
	HMatrixParticle* getPip(Int_t i);
	HMatrixParticle* getPim(Int_t i);
	Int_t getEpN();
	Int_t getEmN();
	Int_t getPN();
	Int_t getPipN();
	Int_t getPimN();


	HGeantKine* getGeantKine(Int_t iTrack, HCategory *pCat=0);

    Float_t shiftPhi(const Float_t phi);

    Bool_t isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2);
	Bool_t isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3);
	Bool_t isUrQMDvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3, const HMatrixParticle *p4);
    Bool_t isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2);
    Bool_t isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3);
    Bool_t isPLUTOvertex(const HMatrixParticle *p1, const HMatrixParticle *p2, const HMatrixParticle *p3, const HMatrixParticle *p4);

	void fillSingle(HMatrixParticle *p, Int_t n, Int_t mult);
	void fillDouble(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2);
	void fillTriple(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2, HMatrixParticle *p3, Int_t n3);
	void fillQuadruple(HMatrixParticle *p1, Int_t n1, HMatrixParticle *p2, Int_t n2, HMatrixParticle *p3, Int_t n3, HMatrixParticle *p4, Int_t n4);


private: 

    Bool_t setAccHistos();
    Bool_t setEffHistos();
    Bool_t setWeightHistos();

    Bool_t isUrQMDprimary(HGeantKine *p);
    Bool_t isPLUTOprimary(HGeantKine *p);


        EXE_PTR sim_execute;
        // Int_t (HMatrixFilter::*sim_execute)();

        TString acc_filename;
        TString effem_filename;
        TString effep_filename;
        TString weight_filename;

        HMatrixHistogram *ep_acc;
        HMatrixHistogram *ep_eff;
        HMatrixHistogram *em_acc;
        HMatrixHistogram *em_eff;
        HMatrixHistogram *p_acc;
        HMatrixHistogram *p_eff;
        HMatrixHistogram *pip_acc;
        HMatrixHistogram *pip_eff;
        HMatrixHistogram *pim_acc;
        HMatrixHistogram *pim_eff;

        HMatrixHistogram *m_weight;
        HMatrixHistogram *pt_weight;
        HMatrixHistogram *y_weight;
 
        
        TFile *file_acc, *file_effep, *file_effem, *file_weight;


        HLinearCategory *pGeantKineCat;

        HIterator *pitGeantKineCat1;
        HIterator *pitGeantKineCat2;

        HGeantKine *pKineHit1;
        HGeantKine *pKineHit2;

        HNtuple *pNt;

        HMatrixParticle *ep;
        HMatrixParticle *em;
        HMatrixParticle *p;
        HMatrixParticle *pip;
        HMatrixParticle *pim;

		std::vector<HMatrixParticle*> epArr;
		std::vector<HMatrixParticle*> emArr;
		std::vector<HMatrixParticle*> pArr;
		std::vector<HMatrixParticle*> pipArr;
		std::vector<HMatrixParticle*> pimArr;
		
		Int_t epN, emN, pN, pipN, pimN;

        HGeomVector vec;
        Float_t ep_mass, em_mass, opening_angle;
        Float_t ep_acc_val, ep_eff_val, em_acc_val, em_eff_val, mass_w_val, pt_w_val, y_w_val;
        TLorentzVector lv2, lv3, lv4;
        TLorentzVector lv_a, lv_b, lv_c, lv_d;

   Float_t mom, theta, phi;
   Float_t momx, momy, momz;
   Int_t parentid, grandparentid, grandgrandparentid, parent, medium, mechanism;
   Float_t geninfo, geninfo1, geninfo2;

   TString tempname;


ClassDef(HMatrixFilter, 0)  //	
};
// ****************************************************************************





