
#include "hparticle.h"
#include "hnparticle.h"
#include <TMath.h>
#include <TLorentzVector.h>

using namespace std;

// ****************************************************************************
ClassImp(HNParticle)

//--------------------------------------------------------------
HNParticle::HNParticle() : id(0), name(), acc(1.), eff(1.)
{
}

//--------------------------------------------------------------
HNParticle::HNParticle(Int_t id, const Char_t* name) : id(id), name(name), acc(0.), eff(0.) 
{
}

//--------------------------------------------------------------
Double_t HNParticle::getOA(Int_t p1, Int_t p2) const
{
   const TLorentzVector *tmp = getLVector(p1);
   return ( TMath::RadToDeg() * tmp->Angle( getLVector(p2)->Vect() ) );
}

//--------------------------------------------------------------
Double_t HNParticle::getOA() const 
{ 
   if (getSize() == 2) return getOA( 0, 1 ); 

return 0.;
} 


//--------------------------------------------------------------
HNParticle::HNParticle(const HParticle& part)
{
   id = part.getId();
   acc = part.getAcc();
   eff = part.getEff();
   name = part.getName();
   vPart = *part.getLVector();
   nPart.push_back( &part );
   opanFilter = kFALSE;
   vertFilter = kFALSE;
   primFilter = kFALSE;
}


//--------------------------------------------------------------
HNParticle& HNParticle::operator+=(const HParticle& part)
{
   id = 100*id + part.getId();
   if (nPart.size() == 0) 
   {
      name = part.getName();
      acc = part.getAcc();
      eff = part.getEff();
   }
   else
   {
      name += string("+");
      name += part.getName();
      acc *= part.getAcc();
      eff *= part.getEff();
   }
   vPart += *part.getLVector();
   nPart.push_back( &part );
   opanFilter = kFALSE;
   vertFilter = kFALSE;
   primFilter = kFALSE;

 return *this;
}

//--------------------------------------------------------------
HNParticle& HNParticle::operator-=(const HParticle& part)
{
   id = 100*id + part.getId();
   if (nPart.size() == 0)
   {
      name = part.getName();
      acc = part.getAcc();
      eff = part.getEff();
   }
   else
   {
      name += string("-");
      name += part.getName();
      acc *= part.getAcc();
      eff *= part.getEff();
   }
   vPart -= *part.getLVector();
   nPart.push_back( &part );
   opanFilter = kFALSE;
   vertFilter = kFALSE;
   primFilter = kFALSE;

 return *this;
}

