#ifndef HNPARTICLE_H
#define HNPARTICLE_H

#include <cassert>
#include <TROOT.h>
#include <TRandom.h>
#include <TLorentzVector.h>
#include <string>
#include <vector>


class HParticle;

// ****************************************************************************


// ****************************************************************************
class HNParticle : public TObject
{
public:

   HNParticle();
   HNParticle(Int_t part, const Char_t* name="");
   ~HNParticle() {}

   void setId(Long_t id) { this->id=id; }
   void setName(const Char_t* name) { this->name=name; }
   void setIdName(Int_t id, const Char_t* name="") { this->id=id; this->name=name; }

   void setAcc(Double_t acc) { this->acc=acc; }
   void setEff(Double_t eff) { this->eff=eff; }

   void setOAstat(Bool_t val) { opanFilter = val; }
   void setVERTEXstat(Bool_t val) { vertFilter = val; }
   void setPRIMARYstat(Bool_t val) { primFilter = val; }

   Bool_t getOAstat() const { return opanFilter; }
   Bool_t getVERTEXstat() const { return vertFilter; }
   Bool_t getPRIMARYstat() const { return primFilter; }

   Long_t getId() const { return id; }
   const Char_t* getName() const { return name.c_str(); }
   Double_t getMass() const { return vPart.M(); }
   Double_t getMass2() const { return vPart.M2(); }
   Double_t getE() const { return vPart.E(); }
   
   Double_t getAcc() const { return acc; }
   Double_t getEff() const { return eff; }
   Double_t getAcc(Int_t i) const { return nPart[i]->getAcc(); }
   Double_t getEff(Int_t i) const { return nPart[i]->getEff(); }

   Double_t getX(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getX(); }
   Double_t getY(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getY(); }
   Double_t getZ(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getZ(); }

   Double_t getP() const { return vPart.P(); }
   Double_t getPt() const { return vPart.Pt(); }
   Double_t getPx() const { return vPart.Px(); }
   Double_t getPy() const { return vPart.Py(); }
   Double_t getPz() const { return vPart.Pz(); }
   Double_t getRapidity() const { return vPart.Rapidity(); }
   Double_t getTheta() const { return vPart.Theta(); }
   Double_t getPhi() const { return vPart.Phi(); }

   Int_t getMedium(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getMedium(); }
   Int_t getMechanism(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getMechanism(); }
   Double_t getGenInfo(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getGenInfo(); }
   Double_t getGenInfo1(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getGenInfo1(); }
   Double_t getGenInfo2(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getGenInfo2(); }
   Int_t getParentTrack(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getParentTrack(); }
   Int_t getParentId(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getParentId(); }
   Int_t getGrandParentId(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getGrandParentId(); }
   Int_t getGrandGrandParentId(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getGrandGrandParentId(); }

   Int_t getSize() const { return nPart.size(); }

   const TLorentzVector* getLVector() const { return &vPart; }
   const TLorentzVector* getLVector(Int_t i) const { assert( i>-1 && i<getSize() ); return nPart[i]->getLVector(); }

   Double_t getOA(Int_t p1, Int_t p2) const;
   Double_t getOA() const; // makes sense only for pair invariant mass

   HNParticle(const HParticle& part);
   HNParticle& operator+=(const HParticle& part);
   HNParticle& operator-=(const HParticle& part);

protected:

   Long_t id;
   std::string name;
   Double_t acc, eff;
   TLorentzVector vPart;

   Bool_t opanFilter; // is oa > 9 for all pairs (value 9 defined in an external filter, may change)
   Bool_t vertFilter; // are all particles from the same vertex
   Bool_t primFilter; // are all particles primary

   std::vector<const HParticle*> nPart;

ClassDef(HNParticle, 0)  //
};
// ****************************************************************************

#endif // HNPARTICLE_H
