// This macro filters data
// Date: 17.11.05 by WP

#include "hparticle.h"
#include "hpidgeanttrackset.h"
#include "hgeantkine.h" 
#include "PParticle.h"
#include <TMath.h>
#include <TLorentzVector.h>

using namespace std;

// ****************************************************************************
ClassImp(HParticle)

HParticle::HParticle() : id(0), name(), acc(0.), eff(0.)
{
	setPxPyPzM();
}

HParticle::HParticle(Int_t id, const Char_t* name) : id(id), name(name), acc(0.), eff(0.) 
{
   switch (id)
   {
       case 2: name = "ep";
	       setPxPyPzM(0., 0., 0., 0.511);
	       break;
       case 3: name = "em";
	       setPxPyPzM(0., 0., 0., 0.511);
	       break;
       case 8: name = "pip";
	       setPxPyPzM(0., 0., 0., 139.569);
	       break;
       case 9: name = "pim";
	       setPxPyPzM(0., 0., 0., 139.569);
	       break;
       case 14: name = "p";
	       setPxPyPzM(0., 0., 0., 938.272);
	       break;
   }
}

HParticle::HParticle(const Char_t* name, double px, double py, double pz) : id(0), name(name), acc(0.), eff(0.)
{
   if ( name == "ep" )
   {
      id = 2;
      setPxPyPzM(px, py, pz, 0.511);
   }
   else if ( name == "em" )
   {
      id = 3;
      setPxPyPzM(px, py, pz, 0.511);
   }
   else if ( name == "pip" )
   {
      id = 8;
      setPxPyPzM(px, py, pz, 139.569);
   }
   else if ( name == "pim" )
   {
      id = 9;
      setPxPyPzM(px, py, pz, 139.569);
   }
   else if ( name == "p" )
   {
      id = 14;
      setPxPyPzM(px, py, pz, 938.272);
   }
}



void HParticle::setPxPyPzM(Double_t px, Double_t py, Double_t pz, Double_t mass)
{
   vPart.SetPxPyPzE(px, py, pz, TMath::Sqrt(mass*mass + px*px + py*py + pz*pz));
}


void HParticle::setPxPyPz(Double_t px, Double_t py, Double_t pz)
{
   vPart.SetPx(px);
   vPart.SetPy(py);
   vPart.SetPz(pz);
}


//--------------------------------------------------------------
HParticle::HParticle(HGeantKine& hit)
{
   id = hit.getID();
   acc = 0.;
   eff = 0.;
   hit.getMomentum(vec);
   switch (id)
   {
       case 2: name = "ep";
               setPxPyPzM(vec.getX(), vec.getY(), vec.getZ(), 0.511);
               break;
       case 3: name = "em";
               setPxPyPzM(vec.getX(), vec.getY(), vec.getZ(), 0.511);
               break;
       case 8: name = "pip";
               setPxPyPzM(vec.getX(), vec.getY(), vec.getZ(), 139.569);
               break;
       case 9: name = "pim";
               setPxPyPzM(vec.getX(), vec.getY(), vec.getZ(), 139.569);
               break;
       case 14: name = "p";
               setPxPyPzM(vec.getX(), vec.getY(), vec.getZ(), 938.272);
               break;
   }
   hit.getCreator(orig.parentTrack, orig.medium, orig.mechanism);
   hit.getGenerator(orig.genInfo, orig.genInfo1, orig.genInfo2);
}



//--------------------------------------------------------------
HParticle::HParticle(const PParticle& hit)
{
   vPart = hit;
   id = hit.ID();
   acc = 0.;
   eff = 0.;
   orig.parentId = hit.getParentId();
   orig.parentTrack = hit.getParentIndex();
   switch (id)
   {
       case 2: name = "ep";
               break;
       case 3: name = "em";
               break;
       case 8: name = "pip";
               break;
       case 9: name = "pim";
               break;
       case 14: name = "p";
               break;
   }

}



//--------------------------------------------------------------
HParticle::HParticle(const HParticle& src)
{
   id = src.id;
   name = src.name;
   acc = src.acc;
   eff = src.eff;
   orig = src.orig;
   vertex = src.vertex;
   vPart = src.vPart;
   vec = src.vec;
}

//--------------------------------------------------------------
HParticle& HParticle::operator=(const HParticle& src)
{
   if (this == &src) return *this;
   id = src.id;
   name = src.name;
   acc = src.acc;
   cout << "AKCEP=: " << acc << endl;
   eff = src.eff;
   orig = src.orig;
   vertex = src.vertex;
   vPart = src.vPart;
   vec = src.vec;

return *this;
}


//--------------------------------------------------------------
HParticle& HParticle::operator+=(const HParticle& part)
{
   id = 100*id + part.id;
   name += "+" + part.name;
   acc *= part.acc;
   eff *= part.eff;
   vPart += part.vPart;

 return *this;
}

//--------------------------------------------------------------
HParticle& HParticle::operator-=(const HParticle& part)
{
   id = 100*id + part.id;
   name += "-" + part.name;
   acc *= part.acc;
   eff *= part.eff;
   vPart -= part.vPart;

 return *this;
}


//--------------------------------------------------------------
HParticle HParticle::operator+(const HParticle& part)
{
   HParticle tmp(*this);
   *this += part;

 return *this;
}

//--------------------------------------------------------------
HParticle HParticle::operator-(const HParticle& part)
{
   HParticle tmp(*this);
   *this -= part;

 return *this;
}

//--------------------------------------------------------------
Double_t HParticle::getTheta(std::string unit) const
{
   if (unit == "deg") return TMath::RadToDeg() * vPart.Theta();
   return vPart.Theta();
}

//--------------------------------------------------------------
Double_t HParticle::getPhi(std::string unit, std::string sector) const
{
   if (unit == "deg")
      if (sector == "one sector")
         return shiftPhi( TMath::RadToDeg() * vPart.Phi() + 180. );
      else
         return TMath::RadToDeg() * vPart.Phi() + 180.;
   return vPart.Phi();
}


//--------------------------------------------------------------
double HParticle::shiftPhi(double phi) const
{
// shift of phi value from 0-360 deg range to one sector 0-60 range
// necessary i.e. for acceptance matrices

   Int_t sector = (Int_t)(phi/60.)-1;   // get sector from phi angle
   if (sector == -1) sector = 5;
   if (sector >= 0 && sector < 5)
   {
      return (phi - (sector+1)*60.); // we shift all to 0-60 deg. range
   }

   return phi;
}

