#ifndef HPOOLFACTORY_H
#define HPOOLFACTORY_H

#include <string>
#include <memory>
#include <TMath.h>
#include "hparticle.h"
#include "hnparticle.h"
#include "hparticles.h"
#include "hparticlepool.h"
#include "hpoolfactoryhandle.h"

using namespace FilteringUtil;

class HPoolFactory 
{
 public:

   HPoolFactory(HParticlePool& ref, std::string name);
   virtual ~HPoolFactory();

   int count();
   virtual void combine() = 0;

   virtual void add(EParticle p1) = 0;
   virtual void add(EParticle p1, EParticle p2) = 0;
   virtual void add(EParticle p1, EParticle p2, EParticle p3) = 0;
   virtual void add(EParticle p1, EParticle p2, EParticle p3, EParticle p4) = 0;

   HNParticle* get(int n = 0) { return &particleSet.at(n); }
   int size() const { return particleSet.size(); }
   virtual bool isMissing() const = 0;

 protected:

   std::string name;
   int number; // all combinatorial possibilities
   int prongNumber; // all particles

   // particleSet contains all combinations (number) making a given entity
   std::vector< HNParticle > particleSet;

   ParticleNum particleComb; // particle_id and number of its combinations (varies from event to event)
   ParticleNum particleNum; // particle_id and number of particles in an entity

   ParticleNumIter particleNumIter;

   HParticlePool& refPool;
   auto_ptr<HPoolFactoryHandle> ptrFH;

};

#endif // HPOOLFACTORY_H
